# frozen_string_literal: true

require 'crep/command'
require 'crep/crash_controller'
require 'crep/model/crash_sources/hockeyapp_crash_source'
require 'crep/model/app_source/hockeyapp_app_source'
require 'hockeyapp'

module Crep
  class Crashes < Command
    DEFAULT_TOP_COUNT = 5
    DEFAULT_TEMPLATE = 'templates/default_crash_report.txt.erb'

    def self.options
      [
        ['--top=5', "If set, Crep will show the top x crashes. #{DEFAULT_TOP_COUNT} by default."],
        ['--identifier=<com.company.app>', 'Crep will show crashes for the app with this bundle identifier'],
        ['--version=<7.10.0>', 'The version of the App.'],
        ['--build=<24>', 'The Build number of the App.'],
        ['--only-unresolved', 'If set, resolved crashes will be filtered out.'],
        ['--template=templates/defaul_crash_report.txt.erb', 'Set this to specify a erb template for the report format. If no value is specifed the default temapate will be used.']
      ].concat(super)
    end

    self.summary = <<-DESC
      Shows a list of crashes
    DESC

    self.description = <<-DESC
      Besides the optional `top`, `template` and `only-unresolved` parameters, Crep requires all other parameters.
    DESC

    def initialize(argv)
      super

      @show_only_unresolved = argv.flag?('only-unresolved', false)
      @top = argv.option('top', DEFAULT_TOP_COUNT)
      @bundle_identifier = argv.option('identifier')
      @version = argv.option('version')
      @build = argv.option('build')
      @template = argv.option('template', DEFAULT_TEMPLATE)
    end

    def validate!
      super

      help! 'The identifier (--identifier) is a required parameter' unless @bundle_identifier
    end

    def run
      crash_datasource = HockeyAppCrashSource.new
      crash_controller = CrashController.new(@bundle_identifier, @top, crash_datasource, @show_only_unresolved, @template)
      if @version && @build
        crash_controller.top_crashes(@version, @build)
      else
        crash_controller.latest_top_crashes
      end
    end
  end
end
